package com.mortals.xhx.module.sst.web;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.Method;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.util.HttpUtil;
import com.mortals.framework.util.StringUtils;
import com.mortals.framework.web.BaseJsonBodyController;
import com.mortals.xhx.common.utils.AESUtil;
import com.mortals.xhx.module.matter.service.UserMatterApplyService;
import com.mortals.xhx.module.sst.feign.IApiComplexFeign;
import com.mortals.xhx.module.sst.pdu.*;
import lombok.Getter;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.net.HttpCookie;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.mortals.xhx.common.key.Constant.PARAM_SERVER_CWS_HTTP_URL;

/**
 * 综窗对接
 */
@RestController
public class SyntheticalController extends BaseJsonBodyController {

    @Value("${upload.path}")
    @Getter
    private String filePath;

    @Autowired
    private IApiComplexFeign iApiComplexFeign;


    /**
     * 查询可受理事项
     * @param query
     * @return
     */
    @PostMapping({"api/event-implementation/list"})
    @UnAuth
    public String eventList(@RequestBody SyntheticalMatterQuery query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/event-implementation/list";
        JSONObject jsonObject = new JSONObject();
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询可受理事项" ;
        try {
            //resp = doPost(url, jsonObject.toJSONStriqng(query));
            resp = iApiComplexFeign.eventList(query);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 查询事项详情
     * @param query
     * @return
     */
    @PostMapping({"api/event-implementation/get-by-id"})
    @UnAuth
    public String eventInfo(@RequestBody SyntheticalPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/event-implementation/get-by-id";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询事项详情" ;
        try {
            //resp = doPost(url, JSONObject.toJSONString(query));
            resp = iApiComplexFeign.eventInfo(query);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 查询事项情形
     * @param query
     * @return
     */
    @PostMapping({"api/bus-situation-material/getSituationMaterialTreeByEventIds"})
    @UnAuth
    public String getSituationMaterialTreeByEventIds(@RequestBody SyntheticalPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/bus-situation-material/getSituationMaterialTreeByEventIds";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询事项情形" ;
        try {
            //resp = doPost(url, JSONObject.toJSONString(query));
            resp = iApiComplexFeign.getSituationMaterialTreeByEventIds(query);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 根据事项EventId查询办理事项材料
     * @param query
     * @return
     */
    @PostMapping({"api/material/findList"})
    @UnAuth
    public String findMaterialList(@RequestBody SyntheticalPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/material/findList";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "根据事项EventId查询办理事项材料" ;
        try {
            //resp = doPost(url, JSONObject.toJSONString(query));
            resp = iApiComplexFeign.findMaterialList(query);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 根据事项情形查询办理事项材料
     * @param query
     * @return
     */
    @PostMapping({"api/consulting-service/getMaterialByEventAndSituation"})
    @UnAuth
    public String getMaterialByEventAndSituation(@RequestBody SyntheticalPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/consulting-service/getMaterialByEventAndSituation";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "根据事项情形查询办理事项材料" ;
        try {
            //resp = doPost(url, JSONObject.toJSONString(query));
            resp = iApiComplexFeign.getMaterialByEventAndSituation(query);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 获取办件要素
     * @param query
     * @return
     */
    @PostMapping({"api/event-implementation/getDynamicFormByEventIds"})
    @UnAuth
    public String getDynamicFormByEventIds(@RequestBody SyntheticalPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/event-implementation/getDynamicFormByEventIds";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "获取办件要素" ;
        try {
            //resp = doPost(url, JSONObject.toJSONString(query));
            resp = iApiComplexFeign.getDynamicFormByEventIds((query));
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 查询字典
     * @param query
     * @return
     */
    @PostMapping({"api/common/queryDict"})
    @UnAuth
    public String queryDict(@RequestBody SyntheticalPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/common/queryDict";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询字典" ;
        try {
            //resp = doPost(url, JSONObject.toJSONString(query));
            resp = iApiComplexFeign.queryDict(query);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 自助设备接件
     * @param pdu
     * @return
     */
    @PostMapping({"api/self-device-info/acceptHandling"})
    @UnAuth
    public String acceptHandling(@RequestBody AcceptHandlingPdu pdu) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/self-device-info/acceptHandling";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "自助设备接件" ;
        try {
            doFileToBase64(pdu);
            String random = "0000000671595991";
            if(ObjectUtil.isNotEmpty(pdu.getApplicantLegal())){
                pdu.getApplicantLegal().setApplicantIdNumber(AESUtil.decryptCBC(pdu.getApplicantLegal().getApplicantIdNumber(),random));
                pdu.getApplicantLegal().setApplicantName(AESUtil.decryptCBC(pdu.getApplicantLegal().getApplicantName(),random));
                pdu.getApplicantLegal().setDocumentId(AESUtil.decryptCBC(pdu.getApplicantLegal().getDocumentId(),random));
                pdu.getApplicantLegal().setPhone(AESUtil.decryptCBC(pdu.getApplicantLegal().getPhone(),random));
            }
            if(ObjectUtil.isNotEmpty(pdu.getApplicantPersonal())){
                pdu.getApplicantPersonal().setApplicantIdNumber(AESUtil.decryptCBC(pdu.getApplicantLegal().getApplicantIdNumber(),random));
                pdu.getApplicantPersonal().setApplicantName(AESUtil.decryptCBC(pdu.getApplicantLegal().getApplicantName(),random));
                pdu.getApplicantPersonal().setDocumentId(AESUtil.decryptCBC(pdu.getApplicantLegal().getDocumentId(),random));
                pdu.getApplicantPersonal().setPhone(AESUtil.decryptCBC(pdu.getApplicantLegal().getPhone(),random));
            }
            //resp = doPost(url, JSONObject.toJSONString(pdu));
            resp = iApiComplexFeign.acceptHandling(pdu);
            log.info("accept resp==>"+resp);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
//            Rest<List<AcceptRspInfo>> rest = JSON.parseObject(resp, new TypeReference<Rest<List<AcceptRspInfo>>>() {
//            });
//            if(rest.getCode()==1){
//                if(CollectionUtils.isNotEmpty(rest.getData())) {
//                    this.userMatterApplyService.saveAcceptRspInfo(rest.getData().get(0), pdu);
//                }
//            }
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 查询已受理事项列表
     * @param pdu
     * @return
     */
    @PostMapping({"api/handling/findPageWithPermission"})
    @UnAuth
    public String findPageWithPermission(@RequestBody FindHandlingPageReq pdu) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/handling/findPageWithPermission";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询已受理事项列表" ;
        try {
            if(StringUtils.isNotEmpty(pdu.getEncryptStr())){
                String random = "0000000671595991";
                pdu.setSearch(AESUtil.decryptCBC(pdu.getEncryptStr(),random));
                log.info(JSONObject.toJSONString(pdu));
            }
            //resp = doPost(url, JSONObject.toJSONString(pdu));
            resp = iApiComplexFeign.findPageWithPermission(pdu);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 查询已受理事项列表
     * @param pdu
     * @return
     */
    @PostMapping({"api/common/downloadFileFtp"})
    @UnAuth
    public String downloadFileFtp(@RequestBody FindHandlingPageReq pdu) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/common/downloadFileFtp";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询已受理事项列表" ;
        try {
            resp = iApiComplexFeign.downloadFileFtp(pdu);
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 将上传文件地址转换为Base64
     * @param pdu
     */
    private void doFileToBase64(AcceptHandlingPdu pdu){
        if(ObjectUtil.isNotEmpty(pdu)){
            List<Material> materials = pdu.getMaterials();
            if(CollectionUtils.isNotEmpty(materials)){
                String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
                for(Material item:materials){
                    if(StringUtils.isNotEmpty(item.getMaterialFilePath())) {
                        String filePath = rootPath + item.getMaterialFilePath();
                        try {
                            Path path = Paths.get(filePath);  // 替换为实际文件路径
                            byte[] fileBytes = Files.readAllBytes(path);
                            String base64EncodedString = Base64.getEncoder().encodeToString(fileBytes);
                            item.setMaterialContent(base64EncodedString);
                        } catch (Exception e) {
                            throw new AppException(e.getMessage());
                        }
                    }
                    if(StringUtils.isNotEmpty(item.getMaterialContent())){
                        log.info("base64");
                        String base64 = String.valueOf(item.getMaterialContent());
                        base64 = base64.replaceAll("data:image/png;base64,","");
                        item.setMaterialContent(base64);
                    }
                }
            }
        }
    }

    private String doPost(String url,String body) throws Exception{
        HttpRequest httpRequest = HttpRequest.post(url);
        httpRequest.body(body);
        HttpResponse execute = httpRequest.execute();

        // 6. 解析这个http响应类，可以获取到响应主体、cookie、是否请求成功等信息
//        boolean ok = execute.isOk(); // 是否请求成功 判断依据为：状态码范围在200~299内
//        System.out.println(ok);
//        List<HttpCookie> cookies = execute.getCookies();// 获取所有cookie
//        cookies.forEach(System.out::println); // 如果为空不会遍历的
        String resp = execute.body();   // 获取响应主体
        return resp;
    }

//    public static void main(String[] args){
//        String url = "http://8.136.255.30:1086/complex/api/event-implementation/get-by-id";
//        JSONObject jsonObject=new JSONObject();
//        System.out.println(jsonObject.toJSONString());
//        jsonObject.put("eventId","2e4575157ed27dbc7330b1595207b49c");
//        String resp = null;
//        Map<String, Object> model = new HashMap();
//        HashMap<String, String> paramsMap = new HashMap<>();
//        try {
//            HttpRequest httpRequest = HttpRequest.post(url);
//            httpRequest.body(jsonObject.toJSONString());
//            HttpResponse execute = httpRequest.execute();
//
//            // 6. 解析这个http响应类，可以获取到响应主体、cookie、是否请求成功等信息
//            boolean ok = execute.isOk(); // 是否请求成功 判断依据为：状态码范围在200~299内
//            System.out.println(ok);
//            List<HttpCookie> cookies = execute.getCookies();// 获取所有cookie
//            cookies.forEach(System.out::println); // 如果为空不会遍历的
//            String body = execute.body();   // 获取响应主体
//            System.out.println(body);
//
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//    }

//    public static void main(String[] args) {
//        try {
//            Path filePath = Paths.get("D:\\home\\mortals\\app\\aaa.jpg");  // 替换为实际文件路径
//            byte[] fileBytes = Files.readAllBytes(filePath);
//            String base64EncodedString = Base64.getEncoder().encodeToString(fileBytes);
//            System.out.println(base64EncodedString);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//    }

}
